import SecurityManager, {
  SYNC_PERPETUAL,
  SYNC_EPHEMERAL
} from "zen-antigen/security";
import { ServiceLocator, noop } from "zen-cerebellum";
import { changeLocale, getServerLocale } from "zen-lingual";
import {
  USER_LOG_OUT,
  USER_SESSION_RESTORE,
  USER_PASSWORD_CHANGE
} from "../constants";
import { historizeUserDisconnection } from "./history";

//simple base 64 encoding
function encodePassword(password) {
  return window.btoa(password);
}

//WAA : change user password
const changePassword = (user, password) => ({
  type: USER_PASSWORD_CHANGE,
  payload: { Password: encodePassword(password) },
  api: {
    endpoint: `/users/${user}/`,
    method: "PATCH"
  }
});

//RAA : reset password of authentified user
export const resetUserPassword = password => (dispatch, getState) => {
  const {
    session: { user }
  } = getState();
  return dispatch(changePassword(user, password));
};

//WAA : check user authentication
const authenticateUser = (Instance, Login, Password = "", remember = false) => {
  return {
    type: "AUTH",
    api: {
      method: "CREATE",
      endpoint: "/auth",
      options: {
        restricted: false
      }
    },
    payload: {
      Login,
      Instance,
      Password: encodePassword(Password),
      remember
    }
  };
};

//RAA : proceed to user login
export const loginUser = (instance, login, password, remember) => {
  return dispatch => {
    SecurityManager.changeSync(remember ? SYNC_PERPETUAL : SYNC_EPHEMERAL);

    return dispatch(authenticateUser(instance, login, password, remember))
      .then(jwt => {
        //init session from token
        dispatch(restoreSession(jwt));
      })
      .fail(() => {
        //force logout when authentication fails to remove any trace of key/token
        disconnectUser();
      });
  };
};

//RAA : proceed to manual user exit
export const logoutUser = () => dispatch => {
  return dispatch(historizeUserDisconnection()).then(() => {
    dispatch(disconnectUser());
  });
};

/** FSA : force current user to be logged out. (to be used by non-public actions)
  @param message : string to display when user logs out
*/
export const disconnectUser = (message = "") => {
  SecurityManager.logout();
  return {
    type: USER_LOG_OUT,
    payload: {
      message
    }
  };
};

/** FSA : restore user from token if provided, otherwise from local storage
  @param token (string) : JWT token
*/
const restoreUser = token => {
  const policy = SecurityManager.init();
  const validToken = policy.restore(token);

  //then sync with session tree
  return {
    type: USER_SESSION_RESTORE,
    payload: {
      auth: validToken && policy.authorized(),
      authMode: policy.authMode,
      user: policy.login,
      instance: policy.instance,
      remember: policy.sync === SYNC_PERPETUAL,
      token: policy.jwt
    }
  };
};

/** RAA : restore session (user + settings) from token if provided, otherwise from local storage
  @param token (string) : JWT token
*/
export const restoreSession = token => {
  //getState is called several times betweeen dispatches to ensure fresh state information.
  return (dispatch, getState) => {
    //restore user first
    dispatch(restoreUser(token));

    //end here if token has not been restored yet because will fail otherwise (restoreSession being call early during initialization flow)
    const state = getState();
    if (state && !state.session.token) return noop();

    //get callback to be fired when user session is restored.
    const sessionCallback = ServiceLocator.get("sessionCallback");
    if (!sessionCallback)
      throw new Error(
        `A session callback declaration is missing in ServiceLocator.`
      );

    //dispatch the callback (action creator expected)
    return dispatch(sessionCallback()).done(() => {
      //retrieve back-office locale selector
      const extractLang = ServiceLocator.get("extractLang") || getServerLocale;
      //change current locale in order to be sync with server one
      dispatch(changeLocale(extractLang(getState())));
    });
  };
};
