import { UPLOAD_DOCUMENT } from '../constants';
import { getEndpoint, getEndpointShort } from './core';
import { computeURL } from '../api/web-api';

/** Compute URLs for document 'actions'
 * @param {object} state - store state tree
 * @param {object} document - document entity model
 * @param {string} mode - (optional) final option mode for URL computation
 * @returns {string} computed URL
 */
const getDocumentURL = (state, document, mode = null) => computeURL(state, `${getEndpoint(document, mode)}`, true);

/** RAA : compute the preview URL of the document
 * @param {object} doc - document entity
 * @return {string} URL of document thumbnail
 */
export function getPreviewURL(doc) {

  return (dispatch, getState) => {

    if (doc) {
      return getDocumentURL(getState(), doc, 'thumbnail');
    }
  };
}

/** RAA - Force the download of the document
 * @param {object} doc - document entity
 */
export function downloadDocument(doc) {

  return (dispatch, getState) => {

    if (doc) {
      const url = getDocumentURL(getState(), doc, '');

      //use URL of download web-service
      window.open(url, '_self', '', true);
    }
  };
}

/** RAA - Display document in browser or PDF viewer
 * @param {object} doc - document entity
 * @param {string} target - target attribute for the new window (_blank, _self)
 * @param {object} options - display options
 *   @param {bool} noViewer - true if embedded ZenPDF viewer shall not be used.
 */
export function displayDocument(doc, target = '_blank', options) {

  return (dispatch, getState) => {

    if (doc) {
      const url = getDocumentURL(getState(), doc, 'preview');
      const { noViewer } = options || false;

      if (noViewer) {
        //force inline content disposition
        window.open(`${url}&disposition=inline`, target, '', true);

      } else {

        //build settings object expected by PDF viewer (json string encoded in base 64)
        let settings = `{"url":"${url}", "name":"${encodeURI(doc.Label)}"}`;
        if (window.btoa) {
          settings = btoa(settings);
        }
        //open external viewer with settings
        window.open(`/pdf/viewer.html?${settings}`, target, '', true);
      }
    }
  };
}

/** RAA - Upload new document to a container entity
 * @param {FileAPI} file - file to upload  https://developer.mozilla.org/fr/docs/Web/API/File
 * @param {object} targetContainer - target entity where to index new document
 * @param {object} params - options parameters to send with uploaded file
 * @param {func} onProgress - callback for progression notification
 */
export function uploadDocument(file, targetContainer, params = {}, onProgress) {
  
  return (dispatch) => {
    if (!file) return;

    //inject file property into form data object
    let formData = new FormData();
    formData.append('file', file);

    //inject all parameters in form
    Object.keys(params).forEach(p => formData.append(p, params[p]));

    //dispatch directly without action creator
    return dispatch({

      type: UPLOAD_DOCUMENT,
      payload: formData,
      api: {
        endpoint: `/upload/${getEndpointShort(targetContainer)}`,
        method: 'CREATE',
        options: {
          onProgress
        }
      }
    });
  };
}